@@ -91,10 +91,10 @@ class MainWindow(hildon.StackableWindow):
         self.series_manager.connect('updated-show-art',
                                     self._update_show_art)
 
-        self.request = AsyncWorker()
-        self.request.queue.put((0, save_pid))
-        self.request.queue.put((0, load_conf_item))
-        self.request.queue.put((0, load_shows_item))
+        self.request = AsyncWorker(True)
+        self.request.queue.put(save_pid)
+        self.request.queue.put(load_conf_item)
+        self.request.queue.put(load_shows_item)
 
         old_pid = self.get_previous_pid()
 
@@ -233,7 +233,7 @@ class MainWindow(hildon.StackableWindow):
         self._have_deleted = True
 
     def _launch_search_shows_dialog(self):
-        search_dialog = SearchShowsDialog(self, self.series_manager)
+        search_dialog = SearchShowsDialog(self, self.series_manager, self.settings)
         response = search_dialog.run()
         show = None
         if response == gtk.RESPONSE_ACCEPT:
@@ -338,9 +338,9 @@ class MainWindow(hildon.StackableWindow):
         save_conf_item = AsyncItem(self.settings.save,
                                (constants.SF_CONF_FILE,),
                                self._save_finished_cb)
-        async_worker = AsyncWorker()
-        async_worker.queue.put((0, save_shows_item))
-        async_worker.queue.put((0, save_conf_item))
+        async_worker = AsyncWorker(False)
+        async_worker.queue.put(save_shows_item)
+        async_worker.queue.put(save_conf_item)
         async_worker.start()
 
     def _save_finished_cb(self, dummy_arg, error):
@@ -589,7 +589,7 @@ class ShowListStore(gtk.ListStore):
 
     def update_pixmaps(self, show = None):
         iter = self.get_iter_first()
-        async_worker = AsyncWorker()
+        async_worker = AsyncWorker(True)
         while iter:
             current_show = self.get_value(iter, self.SHOW_COLUMN)
             same_show = show == current_show
@@ -599,7 +599,7 @@ class ShowListStore(gtk.ListStore):
                                        (current_show, pixbuf),
                                        self._load_pixmap_async_finished,
                                        (current_show,))
-                async_worker.queue.put((0, async_item))
+                async_worker.queue.put(async_item)
                 if same_show:
                     break
             iter = self.iter_next(iter)
@@ -992,6 +992,7 @@ class SeasonSelectView(EnhancedTreeView):
         column = gtk.TreeViewColumn('Name', season_renderer, markup = model.INFO_COLUMN)
         self.set_model(model)
         self.append_column(column)
+        self.get_model().set_sort_func(SeasonListStore.SEASON_COLUMN, self._sort_func)
 
     def set_seasons(self, season_list):
         model = self.get_model()
@@ -1013,7 +1014,7 @@ class SeasonSelectView(EnhancedTreeView):
         season2 = model.get_value(iter2, SeasonListStore.SEASON_COLUMN)
         if season1 == None or season2 == None:
             return 0
-        if season1 < season2:
+        if int(season1) < int(season2):
             return -1
         return 1
 
@@ -1800,13 +1801,13 @@ class FoundShowListStore(gtk.ListStore):
 
 class SearchShowsDialog(gtk.Dialog):
 
-    def __init__(self, parent, series_manager):
+    def __init__(self, parent, series_manager, settings):
         super(SearchShowsDialog, self).__init__(parent = parent)
         self.set_title(_('Search shows'))
 
         self.series_manager = series_manager
         self.series_manager.connect('search-shows-complete', self._search_shows_complete_cb)
-
+        self.settings = settings
         self.connect('response', self._response_cb)
 
         self.chosen_show = None
@@ -1845,9 +1846,11 @@ class SearchShowsDialog(gtk.Dialog):
         self.lang_selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)
         lang_button.set_selector(self.lang_selector)
         try:
-            self.lang_selector.set_active(0, self.series_manager.get_languages().keys().index(self.series_manager.get_default_language()))
+            self.lang_selector.set_active(0, self.series_manager.get_languages().keys().index(self.settings.getConf(Settings.SEARCH_LANGUAGE)))
         except ValueError:
             pass
+        lang_button.connect('value-changed',
+                            self._language_changed_cb)
 
         shows_area = hildon.PannableArea()
         shows_area.add(self.shows_view)
@@ -1862,6 +1865,10 @@ class SearchShowsDialog(gtk.Dialog):
         self.vbox.show_all()
         self.set_size_request(-1, 400)
 
+    def _language_changed_cb(self, button):
+        self.settings.setConf(Settings.SEARCH_LANGUAGE,
+                              self.lang_store[button.get_active()][0])
+
     def _search_entry_changed_cb(self, entry):
         enable = self.search_entry.get_text().strip()
         self.search_button.set_sensitive(bool(enable))
