@@ -24,7 +24,7 @@ from lib import thetvdbapi, serializer, constants
 from lib.util import get_color, image_downloader
 from xml.etree import ElementTree as ET
 from asyncworker import AsyncWorker, AsyncItem
-from lib.constants import TVDB_API_KEY, DATA_DIR, DEFAULT_LANGUAGES
+from lib.constants import TVDB_API_KEY, DATA_DIR, DEFAULT_LANGUAGES, SF_LANG_FILE
 from settings import Settings
 from datetime import datetime
 from datetime import timedelta
@@ -447,7 +447,6 @@ class SeriesManager(gobject.GObject):
             self.series_list = []
 
             self.thetvdb = thetvdbapi.TheTVDB(TVDB_API_KEY)
-            self.async_worker = None
             self.changed = False
             self.auto_save_id = None
 
@@ -456,15 +455,20 @@ class SeriesManager(gobject.GObject):
 
             # Languages
             # self.languages = self.thetvdb.get_available_languages()
-            self.languages = None
+            self.languages = self._load_languages(SF_LANG_FILE)
             self.default_language = None
 
     def emit(self, *args):
         gobject.idle_add(gobject.GObject.emit, self, *args)
 
+    def update_languages(self):
+        self.languages = self.thetvdb.get_available_languages()
+        self._save_languages(SF_LANG_FILE)
+        return self.languages
+
     def get_languages(self):
         if self.languages is None:
-            self.languages = self.thetvdb.get_available_languages()
+            return self.update_languages()
         return self.languages
 
     def get_default_language(self):
@@ -486,15 +490,29 @@ class SeriesManager(gobject.GObject):
 
         return self.default_language
 
+    def _load_languages(self, file_path):
+        if not os.path.exists(file_path):
+            return None
+        return serializer.deserialize(file_path)
+
+    def _save_languages(self, file_path):
+        dirname = os.path.dirname(file_path)
+        if not (os.path.exists(dirname) and os.path.isdir(dirname)):
+            os.mkdir(dirname)
+        serialized = serializer.serialize(self.languages)
+        save_file = open(file_path, 'w')
+        save_file.write(serialized)
+        save_file.close()
+
     def search_shows(self, terms, language = "en"):
         if not terms:
             return []
-        self.async_worker = self.get_async_worker()
+        async_worker = AsyncWorker(True)
         async_item = AsyncItem(self.thetvdb.get_matching_shows,
                                (terms, language,),
                                self._search_finished_callback)
-        self.async_worker.queue.put((0, async_item))
-        self.async_worker.start()
+        async_worker.queue.put(async_item)
+        async_worker.start()
 
     def _search_finished_callback(self, tvdbshows, error):
         shows = []
@@ -509,7 +527,8 @@ class SeriesManager(gobject.GObject):
 
     def update_all_shows_episodes(self, show_list = []):
         show_list = show_list or self.series_list
-        async_worker = self.get_async_worker()
+        async_worker = AsyncWorker(False)
+        update_images_worker = AsyncWorker(True)
         i = 0
         n_shows = len(show_list)
         for i in range(n_shows):
@@ -517,16 +536,16 @@ class SeriesManager(gobject.GObject):
             async_item = AsyncItem(self.thetvdb.get_show_and_episodes,
                                    (show.thetvdb_id, show.language,),
                                    self._set_show_episodes_complete_cb,
-                                   (show, i == n_shows - 1))
-            async_worker.queue.put((0, async_item))
+                                   (show, update_images_worker, i == n_shows - 1))
+            async_worker.queue.put(async_item)
             async_item = AsyncItem(self._set_show_images,
                                    (show,),
                                    None,)
-            async_worker.queue.put((1, async_item))
+            update_images_worker.queue.put(async_item)
         async_worker.start()
         return async_worker
 
-    def _set_show_episodes_complete_cb(self, show, last_call, tvdbcompleteshow, error):
+    def _set_show_episodes_complete_cb(self, show, images_worker, last_call, tvdbcompleteshow, error):
         if not error and tvdbcompleteshow:
             episode_list = [self._convert_thetvdbepisode_to_episode(tvdb_ep,show) \
                             for tvdb_ep in tvdbcompleteshow[1]]
@@ -534,6 +553,7 @@ class SeriesManager(gobject.GObject):
         self.emit(self.UPDATE_SHOW_EPISODES_COMPLETE_SIGNAL, show, error)
         if last_call:
             self.emit(self.UPDATE_SHOWS_CALL_COMPLETE_SIGNAL, show, error)
+            images_worker.start()
 
     def _search_show_to_update_callback(self, tvdbshows):
         if not tvdbshows:
@@ -549,12 +569,12 @@ class SeriesManager(gobject.GObject):
                 break
         if not show_id:
             return
-        self.async_worker = self.get_async_worker()
+        async_worker = AsyncWorker(False)
         async_item = AsyncItem(self._get_complete_show_from_id,
                                (show_id, language,),
                                self._get_complete_show_finished_cb)
-        self.async_worker.queue.put((0, async_item))
-        self.async_worker.start()
+        async_worker.queue.put(async_item)
+        async_worker.start()
 
     def _get_complete_show_from_id(self, show_id, language):
         tvdb_show_episodes = self.thetvdb.get_show_and_episodes(show_id, language)
@@ -577,13 +597,12 @@ class SeriesManager(gobject.GObject):
 
     def _get_complete_show_finished_cb(self, show, error):
         self.emit(self.GET_FULL_SHOW_COMPLETE_SIGNAL, show, error)
-        self.async_worker = self.get_async_worker()
+        async_worker = AsyncWorker(True)
         async_item = AsyncItem(self._set_show_images,
                                (show,),
                                None,)
-        self.async_worker.queue.put((1, async_item))
-        if not self.async_worker.isAlive():
-            self.async_worker.start()
+        async_worker.queue.put(async_item)
+        async_worker.start()
 
     def get_show_by_id(self, show_id):
         for show in self.series_list:
@@ -620,11 +639,6 @@ class SeriesManager(gobject.GObject):
         episode_obj.air_date = thetvdb_episode.first_aired or ''
         return episode_obj
 
-    def stop_request(self):
-        if self.async_worker:
-            self.async_worker.stop()
-            self.async_worker = None
-
     def add_show(self, show):
         if show.id == -1:
             show.id = self._get_id_for_show()
@@ -738,11 +752,6 @@ class SeriesManager(gobject.GObject):
         self.changed = False
         self.emit(self.SHOW_LIST_CHANGED_SIGNAL)
 
-    def get_async_worker(self):
-        if self.async_worker and self.async_worker.isAlive():
-            return self.async_worker
-        return AsyncWorker()
-
     def updated(self):
         self.changed = True
 
